.TH count 1
.SH NAME
count - Generate sequently numbered strings

.SH SYNOPSIS
.B count 
[\-h] 
[\-g\ <file>]
[\-u\ <string>]
[\-t\ <string>] 
[\-p\ <char>[,<int>]]
[\-n]
[\-m\ <string>\ [<string> ...]\ ]
[\-r\ <macro>[,<newmacro>]]
[\-c\ <from>[,<to>[,<step>]]]


.SH DESCRIPTION
.B count
is a program for generating an arbitary number of text strings. These strings
may either be all the same or differ in one or more sequently counted control
variable(s).

.SH OPTIONS
.IP "-h"
Print usage information and exit.

.IP "-n"
Do not use signs for negative numbers, but show them in complement notation 
(representation is platform dependant).

.IP "-c <from>[,<to>[,<step>]]"
Start counting at <from>, stop at <end> and use a stepwidth of <step>. All 
three  arguments must be integers, they may contain negative values. Default 
range is "0,100,1". No output will be generated if either a stepwidth of zero 
is given, or if the argument requests an integer overflow ("-1,1,-1" for
example).

.IP "-t <string>"
Template string for output. Default is "=dec\\en". See section "MACROS" below.

.IP "-g <file>"
Generate output, using the contents of <file> as template.

.IP "-u <string>"
Set userdefined numberbase, accessable via the "=usr" macro. This option is
usefull for example, if lowercase characters are required for hexadecimal 
count. In this case, <string> must be specified as "0123456789abcdef".

.IP "-m <string> [<string> ...]"
Allows to associate a <string> with a number, using the "=map" macro (see
section "MACROS"). This option must be the last one on the commandline and
will override "<to>" of the "-r" switch with the number of <string>s given.

.IP "-p <char>[,<int>]"
Set padding character to <char> and pad all numbers to the length of the
longest number string of the according number base.
If the optional argument is given,
additional <int> padding characters are appended in front of the string.
The default is not to add any leading padding characters. Negative numbers
are shown in complement notation.

.IP "-r <macro>[,<newmacro>]"
Redefine <macro> to <newmacro>. Omitting the replacement results in the
according macro being deleted in the output. 
.BR
Internally <macro> remains
unchanged, therefore it is not possible to daisychain redefinitions, but the
same original macro may be renamed several times.

.SH MACROS
.BR count
prints numbers by using a format string (see -t option). Within this format
string, several macros (all case sensitive) are expanded:

.TP
=bin
The current number in binary notation.

.TP
=oct
The current number in octal notation.

.TP
=dec
The current number in decimal notation.

.TP
=hex
The current number in hexadecimal notation.

.TP
=usr
The current number in userdefined numberbase (see -u flag).

.TP 
=map
The mapping for the current number (see -m flag).

.TP
\\en
A newline character

.TP
\\et
A tabulator character


.SH ENVIRONMENT

.IP FSTRING
The contents of the environment variable FSTRING will taken as format
string, if set and not overridden by the "-t" or "-g" commandline switch.


.SH EXAMPLES
This section contains some examples, to illustrate the use the count program.

.IP "Cleaning up a messed up image dir"
count -c 10,0,10 -t "mv =map image-=dec.jpg\\en" -p 0,1 -m *.JPG | bash

.IP "Creating/Deleting two dozen systemaccounts in a jiffy"

count -c 500,524,1 -t "useradd -m /home/user=dec -u =dec user=dec | bash


count -c 500,524,1 -t "userdel -r user=dec\\en" | bash

.SH AUTHOR
Patrick Ahlbrecht <p.ahlbrecht@billiton.de>
