/*
    Copyright (C) 2003, Patrick Ahlbrecht

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "util.h"
#include "config.h"


void setDefaults() {
  cfg.delim=',';
  cfg.tmplfile=NULL;
  cfg.miss="";
  cfg.empty="";
  cfg.opts=OPT_NONE;
  cfg.fields=(char**)calloc(1,sizeof(char*));
  oomtest(cfg.fields);
}

void putEntry(const char* key, char* val) {
  if (key==NULL || val==NULL) return;

  if(!strcasecmp(key,"options")) { 
    int i=0;
    char** tmp=splitString(val,-1,' ');
    while(tmp[i]!=NULL) {
      if (!strcasecmp(tmp[i],"strip")) cfg.opts=cfg.opts|OPT_STRIP;
      if (!strcasecmp(tmp[i],"singlelined")) cfg.opts=cfg.opts|OPT_NOFOLD;
      if (!strcasecmp(tmp[i],"behead")) cfg.opts=cfg.opts|OPT_IGHEADER;
      if (!strcasecmp(tmp[i],"nolinebreak")) cfg.opts=cfg.opts|OPT_NOBREAK;
      i++;
    }
    freeList((void**)tmp);
    free(val);
    return;
  }
  
  if(!strcasecmp(key,"separator")) {
    cfg.delim=val[0];
    free(val);
    return;
  }
  
  if(!strcasecmp(key,"fields")) {
    cfg.fields=splitString(val,-1,' ');
    free(val);
    return;
  }
  
  if(!strcasecmp(key,"template")) {
    cfg.tmplfile=val;
    return;
  }
  
  if(!strcasecmp(key,"missing")) {
    cfg.miss=val;
    return;
  }
  
  if(!strcasecmp(key,"empty")) {
    cfg.empty=val;
    return;
  }
  
  fprintf(stderr,"Unknown config directive: %s\n",key);
}

int readConf(const char* fname) {
  FILE* fptr;
  char buf[MAXLINE];
  char** tmp;
  int i=0;
  if (fname==NULL) {
    fprintf(stderr,"Cannot operate without a control file\n");
    exit(EXIT_FAILURE);
  }
  
  if ((fptr=fopen(fname,"r"))==NULL) {
    perror(fname);
    exit(EXIT_FAILURE);
  }
  
  setDefaults();
  
  while(((fgets(buf, (size_t)sizeof(buf), fptr)) && (buf != NULL))) {
    if (!((buf[0]=='#') || (*buf=='\n'))) {
      if (buf[strlen(buf)-1]=='\n') buf[strlen(buf)-1]='\0';
      tmp=splitString(buf,1,' ');
      putEntry(tmp[0],tmp[1]);
      free(tmp[0]);
    }
  }
  (void)fclose(fptr);
  
  return 0;
}
