/*
    Copyright (C) 2003, Patrick Ahlbrecht

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <getopt.h>
#include <stdio.h>
#include <string.h>
#include "util.h"

#define MAXLINE 65535

// Set program version
char* version=VERSION;

/**
 * Fill the inputbuffer
 */
char* readFromStdIN() {
  char ibuf[MAXLINE];
  int len=0;
  char* buffer=NULL;
  
  while (fgets(ibuf,(int)sizeof(ibuf)-1,stdin)) {
    if (buffer!=NULL) len=strlen(buffer);
    buffer=(char*)realloc(buffer,(len+strlen(ibuf)+1)*sizeof(char));
    oomtest(buffer);
    memcpy(buffer+len,ibuf,strlen(ibuf)+sizeof(char));
  }
  return buffer;
}

/**
 * Rotate input
 * @param txt text to rotate
 * @param alph alphabet to rotate through. This is a NULL terminated list of
 * charactersets. Only characters in the same characterset are replaced;
 * @param rot rotation factor
 */
char* rotate(char* txt, const char** alph, const int rot,int from, int to) {
  int x=0;
  int y=0;
  
  while(alph[x]!=NULL) {
    for(y=from;y<to;y++) {
      int z=isIn(alph[x],txt[y]);
      if (z!=-1) {
        
        txt[y]=alph[x][(z+rot)%strlen(alph[x])];
      }
    }
    x++;
  }
}


/**
 * Print Usage information and exit
 * @param name progname (argv[0] of main)
 */
printUsage(const char* name) {
  char* tmp=(char*)basename((char*)name);
  
  printf("%s [OPTIONS] ...\n\
  \nrot13 - scramble ASCII text\n\n\
  -h\t\t\t\tPrint help\n\
  -r <int>\t\t\tRotate by <int> characters instead of 13\n\
  -f <file>\t\t\tRead text to rotate from file <file>\n\
  -a <string> [<string> ...]\tSpecify different alphabet\n\
  -o <file>\t\t\tWrite output to <file>\n\
  -p <from>[,to]\t\tOnly encode a passage of the text\n\
  \n",tmp);
  
  exit(EXIT_SUCCESS);
}

int main(int argc,char** argv) {
  int ch;
  int rot=13;
  char* buffer=NULL;
  char** alph=NULL;
  FILE* out=stdout;
  int from=0;
  int to=-1;
  
  while ((ch = getopt(argc, argv, "hf:r:a:o:p:")) !=-1) {
    if (alph!=NULL) {
      printf("\"-a\" must be specified last!\n");
      return EXIT_FAILURE;
    }
    
    switch(ch) {
      case 'h': {
        printUsage(argv[0]);
        break;
      }
      
      case 'f': {
        if (buffer!=NULL) break;
        buffer=(char*)readFile(optarg);
        break;
      }
      
      case 'r' : {
        rot=abs(atoi(optarg));
        break;
      }
      
      case 'a' : {
        int i;
        for (i=1;i<argc;i++) if (!strcasecmp(argv[i],"-a")) break;
        alph=(char**)calloc((argc+1),sizeof(char*));
        oomtest(alph);
        memcpy(alph,argv+i+1,(argc-i)*sizeof(char*));
        break;
      }
      case 'o': {
        out=fopen(optarg,"w");
        if(out==NULL) {
          perror(optarg);
          return EXIT_FAILURE;
        }
        break;
      }
      case 'p': {
        char** tmp=splitString(optarg,1,',');
        from=atoi(tmp[0]);
        if (tmp[1]!=NULL) to=atoi(tmp[1]);
        break;
      }
      default: {printUsage(argv[0]);}
    }
  }
  
  
  if(alph==NULL) {
    alph=(char**)calloc(3,sizeof(char*));
    oomtest(alph);
    alph[0]="abcdefghijklmnopqrstuvwxyz";
    alph[1]="ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  }
  
  if (buffer==NULL) buffer=readFromStdIN();
  
  if (buffer!=NULL && alph!=NULL) {
    if (from<0) from=0;
    if (to<0 || to>strlen(buffer)) to=strlen(buffer);
    rotate(buffer,(const char**)alph,rot,from,to);
    fprintf(out,buffer);
    return EXIT_SUCCESS;
  }
  return EXIT_FAILURE;
}
