/*
    Copyright (C) 2003, Patrick Ahlbrecht

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>
#include "util.h"

void oomtest(void* testvar) {
  if(testvar==NULL) {
    printf("Error allocating memory\n");
    exit(EXIT_FAILURE);
  }
}

char* cpyStr(const char* src) {
  int i=0;
  char* dest;
  
  if(src==NULL) {
    dest=(char*)calloc(1,sizeof(char));
    oomtest(dest);
    return dest;
  }
  
  dest=(char*)malloc((strlen(src)+1)*sizeof(char));
  oomtest(dest);
  
  while(src[i]!='\0') {
    dest[i]=src[i];
    i++;
  }
  dest[i]='\0';
  return dest;
}


char* expand(char* txt, const char* s, const char *r) {
  int x=0;
  int y=0;
  char* tmp;
  
  if ((txt==NULL) || (s==NULL) || s[0]=='\0') return txt;
  
  while(x<strlen(txt)) {
    for(y=0;y<strlen(s) && x+y<strlen(txt);y++) { if (txt[x+y]!=s[y]) break;}

    if (y==strlen(s)) {
      if (r==NULL) {
        memmove(txt+x,txt+x+y,strlen(txt)-x-y+sizeof(char));
      }
      else {
        txt=(char*)realloc(txt,(strlen(txt)+strlen(r)+1)*sizeof(char));
        oomtest(txt);
        memmove(txt+x+strlen(r),txt+x+y,strlen(txt)-x-y+sizeof(char));
        memcpy(txt+x,r,strlen(r));
        x=x+strlen(r);
      }
    }
    else x++;
  }
  return txt;
}

char **splitString(const char* str, int idx, char delim) {
  int x=0;
  int y=0;
  int z=0;
  int len;
  char* tmp;
  char** ret;
  
  for(x=0;x<strlen(str);x++) {
    while(str[x]==delim && x<(int)strlen(str)) x++;
    while(str[x]!=delim && x<(int)strlen(str)) x++;
    z++;
  }
  
  z++;
  ret=(char**)calloc((size_t)z,sizeof(char*));
  oomtest(ret);
  z=0;
  
  len=strlen(str);
  while(str[len-1]==delim) len--;
  
  for(x=0;x<len;x++) {
    while(str[x]==delim && x<len) x++;
    y=x;
    if(z==idx && idx>-1) x=len;
    while(str[x]!=delim && x<len) x++;
    
    tmp=(char*)calloc((size_t)x-y+3,sizeof(char));
    oomtest(tmp);
    memcpy(tmp,str+y*sizeof(char),(size_t)x-y);
    ret[z]=tmp;
    z++;
  }
  return ret;
}

void freeList(void** rel) {
  int i=0;
  if (rel==NULL) return;
  while(rel[i]!=NULL) {
    free(rel[i]);
    rel[i]=NULL;
    i++;
  }
  free(rel);
  rel=NULL;
}

char* readFile(const char* fname) {
  int fd;
  struct stat fs;
  char* buf;

  fd=open(fname,O_RDONLY);
  if (fd==-1) return NULL;
  if (fstat(fd,&fs)==-1) return NULL;
  
  buf=(char*)calloc((size_t)fs.st_size+1,sizeof(char));
  oomtest(buf);
  if (fs.st_size!=read(fd,buf,(size_t)fs.st_size)) return NULL;
  
  return buf;
}

/**
 * Convert integer to string
 * @param i the number to convert
 * @param digits a NULL terminated array of characters, containing all
 * digits of the numberbase( must be orderes by value, eg: To select dezimal
 * output, give it {'0','1','2','3','4','5','6','7','8','9',(char)NULL } ).
 * @return The integer as ascii string. Note: as the numberbase is freely 
 * selectable, itos() doesn't do any formatig (like prefixing hexnumbers with
 * "0x"). Negative numbers are not prefixed by a sign, but complement encoded.
 * A NULL string will be returned, if no string can be computed.
 */
char* itos(const unsigned long int i, const char* digits){
  int base=0;
  char* res;
  unsigned long int a,b,c=0;
  
  if(digits==NULL || strlen(digits)<=1) return NULL;

  while(digits[base]!='\0') base++;
  
  b=i;
  
  do { c++;b/=base;} while(b!=0);

  res=(char*)calloc(c+1,sizeof(char));
  oomtest(res);
  
  b=i;
  for(a=c;a>0;a--) {
    if (b==0 && c>1) break;
    res[a-1]=digits[b%base];
    b=b/base;
  }
  return res;
}


int isIn(const char* set, const char test) {
  int i=0;
  for (i=0;i<=strlen(set);i++) {
    if (set[i]==test) return i;
  }
  return -1;
}

char* cpyTxt(const char* txt, const int beg, const int end) {
  char* tmp;
  int i=0;
  
  if(beg>=end || txt==NULL) return NULL;
    
  tmp=(char*)calloc((end-beg)+1,sizeof(char));
  oomtest(tmp);
  
  while(i<end-beg) {
    tmp[i]=txt[beg+i];
    i++;  
  }
  return tmp;
}
