#ifndef UTIL_H
#define UTIL_H

/**
 * Always useful
 */
#define TRUE 1
#define FALSE 0

/**
 * Tests, if a variable is allocated (call after malloc/calloc/realloc)
 */
void oomtest(void* testvar);

/**
 * Replace variables in text.
 * @param txt the text buffer. This must not be a constant string!
 * @param s string representing the variable
 * @param r replacement for s
 * If txt or s is NULL, this function will simply return, if r is null, s
 * will be cut out of txt. the substitution string may contain the original
 * string.
 * @return a pointer to the expanded text
 */
char* expand(char* txt, const char* s, const char* r);

/**
 * Split a string into tokens.
 * @param str the string to split
 * @param idx stop tokenization after finding idx tokens (the rest of the
 * string will be put into the last token, no matter how many whitespaces it 
 * may contain). A value of 0 means: return the complete string, while -1
 * stands for unlimited tokens.
 * @param delim char to
 * @return a NULL terminated array with substrings of str
 */
char **splitString(const char*, int idx, char delim);

/**
 * Convienience method to free null terminated arrays
 * @param rel a pointer to a NULL terminated aray of somethings. Every
 * element of rel (and rel itself) will be freed. and set to NULL
 */
void freeList(void** rel);

/**
 * Make a copy of a string 
 * @param src source string
 * @return a pointer to dest
 */
char* cpyStr(const char* src);

/**
 * Read a file
 * @param fname filename
 * @return contents of the file or NULL
 */
char* readFile(const char* fname);

/**
 * Convert integer to string
 * @param i the number to convert
 * @param digits a NULL terminated array of characters, containing all
 * digits of the numberbase( must be orderes by value, eg: To select dezimal
 * output, give it {'0','1','2','3','4','5','6','7','8','9',(char)NULL } ).
 * @return The integer as ascii string. Note: as the numberbase is freely 
 * selectable, itos() doesn't do any formatig (like prefixing hexnumbers with
 * "0x"). Negative numbers are not prefixed by a sign, but complement encoded.
 * A NULL string will be returned, if no string can be computed.
 */
char* itos(const unsigned long int i, const char* digits);

/**
 * Determine, whether a certain character is contained in a certain charset
 * @param set charset (NULL terminated string)
 * @param test character to test
 * @return the index of test in set or -1 if not found
 */
int isIn(const char* set, const char test);

/**
 * Copies a portion of a txt string
 * @param txt the complete text
 * @param beg first character
 * @param end last character
 * @return a copy of the text between beg and end. 
 * Or NULL, if (txt==NULL || beg>=end)
 */
char* cpyTxt(const char* txt, const int beg, const int end);


#endif
